@@ -6,12 +6,12 @@ import json
 import os
 import sys
 import popUpAlgoFollow
+import pandas as pd
 
 import numpy as np
 from filterpy.kalman import KalmanFilter
 from filterpy.common import Q_discrete_white_noise
 
-# from calculateCurvature import calculateCurvature
 from detectMovementWithRawVideo import detectMovementWithRawVideo
 from getTailTipManual import getHeadPositionByFileSaved, getTailTipByFileSaved
 
@@ -60,16 +60,16 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
   thresAngleBoutDetect = hyperparameters["thresAngleBoutDetect"]
   debugExtractParams = hyperparameters["debugExtractParams"]
 
-  trackingTailAllAnimals = trackingData[0]
+  trackingHeadTailAllAnimals = trackingData[0]
   trackingHeadingAllAnimals = trackingData[1]
-  headPosition = trackingData[2]
-  tailTip = trackingData[3]
+  headPositionFirstFrame = trackingData[2]
+  tailTipFirstFrame = trackingData[3]
   
   data = []
   
-  for animalId in range(0, len(trackingTailAllAnimals)):
+  for animalId in range(0, len(trackingHeadTailAllAnimals)):
     
-    trackingTail    = trackingTailAllAnimals[animalId]
+    trackingTail    = trackingHeadTailAllAnimals[animalId]
     trackingHeading = trackingHeadingAllAnimals[animalId]
     
     n = len(trackingTail[0])
@@ -87,22 +87,6 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
     tailX   = np.zeros((nbFrames, n))
     tailY   = np.zeros((nbFrames, n))
     
-    # fTipX = KalmanFilter (dim_x=2, dim_z=1)
-    # fTipX.x = np.array([[2.],[0.]])
-    # fTipX.F = np.array([[1.,1.],[0.,1.]])
-    # fTipX.H = np.array([[1.,0.]])
-    # fTipX.P = np.array([[1000.,    0.],[   0., 1000.] ])
-    # fTipX.R = np.array([[5.]])
-    # fTipX.Q = Q_discrete_white_noise(dim=2, dt=0.1, var=0.13)
-
-    # fTipY = KalmanFilter (dim_x=2, dim_z=1)
-    # fTipY.x = np.array([[2.],[0.]])
-    # fTipY.F = np.array([[1.,1.],[0.,1.]])
-    # fTipY.H = np.array([[1.,0.]])
-    # fTipY.P = np.array([[1000.,    0.],[   0., 1000.] ])
-    # fTipY.R = np.array([[5.]])
-    # fTipY.Q = Q_discrete_white_noise(dim=2, dt=0.1, var=0.13)
-    
     if hyperparameters["headingCalculationMethod"] == "calculatedWithMedianTailTip":
       tip2 = np.zeros((nbFrames, 2))
       nbTailPoints = len(trackingTail[0]) - 1
@@ -122,18 +106,6 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       nbTailPoints = len(trackingTail[0]) - 1
       tip[i]     = np.array([ trackingTail[i][nbTailPoints][0], trackingTail[i][nbTailPoints][1] ])
       
-      # print("bef:",tip[i])
-      # fTipX.predict()
-      # fTipX.update(np.array([tip[i][0]]))
-      # tip[i][0] = fTipX.x[0]
-      # trackingTail[i][nbTailPoints][0] = fTipX.x[0]
-      
-      # fTipY.predict()
-      # fTipY.update(np.array([tip[i][1]]))
-      # tip[i][1] = fTipY.x[0]
-      # trackingTail[i][nbTailPoints][1] = fTipY.x[0]
-      # print("aft:",tip[i])
-      
       head[i]    = np.array([ trackingTail[i][0][0], trackingTail[i][0][1] ])
       
       if hyperparameters["headingCalculationMethod"] == "calculatedWithFirstTailPt":
@@ -153,15 +125,6 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       
       tailX[i]   = trackingTail[i,:,0]
       tailY[i]   = trackingTail[i,:,1]
-
-      # if False:
-        # tailForCurv = np.zeros((10, 2))
-        # for j in range(0,10):
-          # tailForCurv[j][0] = tailX[i][j]
-          # tailForCurv[j][1] = tailY[i][j]
-        # curvature = calculateCurvature(tailForCurv)
-        # diff = np.max(np.abs(curvature))
-        # curvatureBef = curvature
     
     if hyperparameters["noBoutsDetection"] == 1:
       auDessus        = np.zeros((nbFrames, 1))
@@ -177,7 +140,22 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
             auDessus[i] = 1
       else:
         if hyperparameters["thresForDetectMovementWithRawVideo"] == 0:
-          window = 5
+          
+          # Calculating angle median
+          angle2 = np.transpose(angle)
+          angle2 = angle2[0]
+          rolling_window = hyperparameters["tailAngleMedianFilter"]
+          if rolling_window > 0:
+            shift = int(-rolling_window / 2)
+            angle_median = np.array(pd.Series(angle2).rolling(rolling_window).median())
+            angle_median = np.roll(angle_median, shift)
+            for ii in range(0, rolling_window):
+              angle_median[ii] = angle2[ii]
+            for ii in range(len(angle_median)-rolling_window,len(angle_median)):
+              angle_median[ii] = angle2[ii]
+          else:
+            angle_median = angle2
+          # Calculating angle variation to detect movement
           angleVariation  = np.zeros((nbFrames, 1))
           auDessus        = np.zeros((nbFrames, 1))
           for i in range(0,nbFrames):
@@ -185,19 +163,21 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
               print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
             min = 10000
             max = -10000
-            for j in range(i-2*window,i+1):
+            # for j in range(i-hyperparameters["windowForBoutDetectWithAngle"], i+1):
+            for j in range(i-int(hyperparameters["windowForBoutDetectWithAngle"]/2), i+int(hyperparameters["windowForBoutDetectWithAngle"]/2)):
               if (j >= 0) and (j<nbFrames):
-                if angle[j] > max:
-                  max = angle[j]
-                if angle[j] < min:
-                  min = angle[j]
+                if angle_median[j] > max:
+                  max = angle_median[j]
+                if angle_median[j] < min:
+                  min = angle_median[j]
             angleVariation[i] = max - min
             if debug:
               print(i,angleVariation[i])
             if angleVariation[i] > thresAngleBoutDetect:
               auDessus[i] = 1
+          
         else:
-          auDessus = detectMovementWithRawVideo(hyperparameters, videoPath, background, wellNumber, wellPositions, head, headPosition, tailTip)
+          auDessus = detectMovementWithRawVideo(hyperparameters, videoPath, background, wellNumber, wellPositions, head, headPositionFirstFrame, tailTipFirstFrame)
     
     if (hyperparameters["freqAlgoPosFollow"] != 0):
       print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
@@ -236,7 +216,7 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       if (position == 1) and ((auDessus2[i] == 0) or (i == nbFrames-1)):
         curBout[0][2] = i-1
         position = 0
-        if curBout[0][2] - curBout[0][1] >= hyperparameters["detectBoutMinNbFrames"] or hyperparameters["noChecksForBoutSelectionInExtractParams"]:
+        if curBout[0][2] - curBout[0][1] >= hyperparameters["detectBoutMinNbFrames"] or (hyperparameters["noChecksForBoutSelectionInExtractParams"] and curBout[0][2] - curBout[0][1] >= 1):
           debMouv = int(curBout[0][1])
           endMouv = int(curBout[0][2])
           dist = math.sqrt( (head[debMouv,0]-head[endMouv,0])**2 + (head[debMouv,1]-head[endMouv,1])**2 )
@@ -272,18 +252,15 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       item = {}
       start = int(bouts[i][1])
       end   = int(bouts[i][2])
-      item["AnimalNumber"]  = animalId #int(bouts[i][0])
+      item["AnimalNumber"]  = animalId
       item["BoutStart"]     = start + firstFrame
       item["BoutEnd"]       = end + firstFrame
       item["TailAngle_Raw"] = angle[start:end+1,0].tolist()
       item["HeadX"]         = head[start:end+1,0].tolist()
       item["HeadY"]         = head[start:end+1,1].tolist()
-      # item["Heading_raw"]   = heading[start:end,0]
       item["Heading"]       = heading[start:end+1,0].tolist()
       item["TailX_VideoReferential"] = tailX[start:end+1].tolist()
       item["TailY_VideoReferential"] = tailY[start:end+1].tolist()
-      # item["TailX_HeadingReferential"]
-      # item["TailY_HeadingReferential"]
       data.append(item)
   
   print("Parameters extracted for well",wellNumber)
