@@ -3,21 +3,21 @@ import cv2
 import numpy as np
 from calculateHeading import calculateHeadingSimple
 
-def calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, output, i, firstFrame):
+def calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, trackingHeadTailAllAnimals, i, firstFrame):
   mindist   = 1000000
   for animal_Id2 in range(0, hyperparameters["nbAnimalsPerWell"]):
-    x_curFrame_animal_Id   = output[animal_Id, i-firstFrame][0][0]
-    y_curFrame_animal_Id   = output[animal_Id, i-firstFrame][0][1]
+    x_curFrame_animal_Id   = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+    y_curFrame_animal_Id   = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
     if x_curFrame_animal_Id != 0 and y_curFrame_animal_Id != 0:
       if animal_Id != animal_Id2:
-        x_curFrame_animal_Id2  = output[animal_Id2, i-firstFrame][0][0]
-        y_curFrame_animal_Id2  = output[animal_Id2, i-firstFrame][0][1]
+        x_curFrame_animal_Id2  = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][0]
+        y_curFrame_animal_Id2  = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][1]
         dist = math.sqrt((x_curFrame_animal_Id-x_curFrame_animal_Id2)**2 + (y_curFrame_animal_Id-y_curFrame_animal_Id2)**2)
         if dist < mindist:
           mindist = dist
   return mindist
 
-def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, gray, i, firstFrame, thresh1, thresh3, lengthX):
+def multipleAnimalsHeadTrackingAdvance(trackingHeadingAllAnimals, trackingHeadTailAllAnimals, hyperparameters, gray, i, firstFrame, thresh1, thresh3, lengthX):
   
   marginX = hyperparameters["multipleAnimalTrackingAdvanceAlgorithmMarginX"]
   
@@ -109,10 +109,10 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
     while animalNotPutOrEjectedBecausePositionAlreadyTaken:
       animalNotPutOrEjectedBecausePositionAlreadyTaken = 0
       for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-        x_prevFrame_animal_Id = output[animal_Id, i-firstFrame-1][0][0]
-        y_prevFrame_animal_Id = output[animal_Id, i-firstFrame-1][0][1]
-        x_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][0]
-        y_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][1]
+        x_prevFrame_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][0]
+        y_prevFrame_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][1]
+        x_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+        y_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
         if ((x_prevFrame_animal_Id != 0) and (y_prevFrame_animal_Id != 0) and (x_curFrame_animal_Id == 0) and (y_curFrame_animal_Id == 0)):
           min_dist  = 10000000000000000000000
           index_min = -1
@@ -130,8 +130,8 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
           if index_min >= 0:
             if (headCoordinatesOptionsAlreadyTakenDist[index_min] == -1):
               # Save position of animal for frame i-firstFrame
-              output[animal_Id, i-firstFrame][0][0] = headCoordinatesOptions[index_min][0]
-              output[animal_Id, i-firstFrame][0][1] = headCoordinatesOptions[index_min][1]     
+              trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = headCoordinatesOptions[index_min][0]
+              trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = headCoordinatesOptions[index_min][1]     
               headCoordinatesOptionsAlreadyTakenDist[index_min]     = min_dist
               headCoordinatesOptionsAlreadyTakenAnimalId[index_min] = animal_Id
             else:
@@ -139,11 +139,11 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
               if min_dist < headCoordinatesOptionsAlreadyTakenDist[index_min]:
                 # "Ejecting" previously saved position of animal for frame i-firstFrame
                 animal_Id_Eject = headCoordinatesOptionsAlreadyTakenAnimalId[index_min]
-                output[animal_Id_Eject, i-firstFrame][0][0] = 0
-                output[animal_Id_Eject, i-firstFrame][0][1] = 0          
+                trackingHeadTailAllAnimals[animal_Id_Eject, i-firstFrame][0][0] = 0
+                trackingHeadTailAllAnimals[animal_Id_Eject, i-firstFrame][0][1] = 0          
                 # Replace position of animal for frame i-firstFrame
-                output[animal_Id, i-firstFrame][0][0] = headCoordinatesOptions[index_min][0]
-                output[animal_Id, i-firstFrame][0][1] = headCoordinatesOptions[index_min][1]  
+                trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = headCoordinatesOptions[index_min][0]
+                trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = headCoordinatesOptions[index_min][1]  
                 headCoordinatesOptionsAlreadyTakenDist[index_min]     = min_dist
                 headCoordinatesOptionsAlreadyTakenAnimalId[index_min] = animal_Id               
   
@@ -151,24 +151,24 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
   
   for idxCoordinateOption, [idx_x_Option, idx_y_Option] in enumerate(headCoordinatesOptions):
     for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-      if (output[animal_Id, i-firstFrame][0][0] == 0) and (output[animal_Id, i-firstFrame][0][1] == 0):
+      if (trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] == 0) and (trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] == 0):
         if i == firstFrame:
-          output[animal_Id, i-firstFrame][0][0] = idx_x_Option
-          output[animal_Id, i-firstFrame][0][1] = idx_y_Option
+          trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = idx_x_Option
+          trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = idx_y_Option
           break
         else:
-          if (output[animal_Id, i-firstFrame-1][0][0] == 0) and (output[animal_Id, i-firstFrame-1][0][1] == 0): # NEW !!!!
-            output[animal_Id, i-firstFrame][0][0] = idx_x_Option
-            output[animal_Id, i-firstFrame][0][1] = idx_y_Option
+          if (trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][0] == 0) and (trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][1] == 0): # NEW !!!!
+            trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = idx_x_Option
+            trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = idx_y_Option
             break
             
   # Rescuing disparitions in the middle of the thing
   if i > firstFrame:
     for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-      x_prevFrame_animal_Id = output[animal_Id, i-firstFrame-1][0][0]
-      y_prevFrame_animal_Id = output[animal_Id, i-firstFrame-1][0][1]
-      x_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][0]
-      y_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][1]
+      x_prevFrame_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][0]
+      y_prevFrame_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-1][0][1]
+      x_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+      y_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
       if ((x_prevFrame_animal_Id != 0) and (y_prevFrame_animal_Id != 0) and (x_curFrame_animal_Id == 0) and (y_curFrame_animal_Id == 0) and (x_prevFrame_animal_Id > marginX) and (x_prevFrame_animal_Id < lengthX - marginX) ):
         print("frame:", i, " ; animal:", animal_Id, " has disapeared, looking for alternative")
         min_dist  = 10000000000
@@ -179,11 +179,11 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
             min_dist = dist
             index_min = idxCoordinateOption
         print("index_min:", index_min)
-        mindistFromOtherAnimals = calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, output, i, firstFrame)
+        mindistFromOtherAnimals = calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, trackingHeadTailAllAnimals, i, firstFrame)
         if index_min != -1 and mindistFromOtherAnimals >= 5:
           print("coord:", headCoordinatesOptionsAlt[index_min][0], headCoordinatesOptionsAlt[index_min][1], headCoordinatesOptionsAlt[index_min][2], headCoordinatesOptionsAlt[index_min][3], headCoordinatesOptionsAlt[index_min][4], headCoordinatesOptionsAlt[index_min][5])
-          output[animal_Id, i-firstFrame][0][0] = headCoordinatesOptionsAlt[index_min][0]
-          output[animal_Id, i-firstFrame][0][1] = headCoordinatesOptionsAlt[index_min][1]
+          trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = headCoordinatesOptionsAlt[index_min][0]
+          trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = headCoordinatesOptionsAlt[index_min][1]
           # THIS BELLOW IS NEW !!!
           if headCoordinatesOptionsAlt[index_min][2] != -1: # this alt head option is coming from a blob split in half
             for animal_Id2 in range(0, hyperparameters["nbAnimalsPerWell"]):
@@ -194,12 +194,12 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
               alt_opo_x      = headCoordinatesOptionsAlt[index_min][4]
               alt_opo_y      = headCoordinatesOptionsAlt[index_min][5]
               if animal_Id2 != animal_Id:
-                otherAnimal_x = output[animal_Id2, i-firstFrame][0][0]
-                otherAnimal_y = output[animal_Id2, i-firstFrame][0][1]
+                otherAnimal_x = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][0]
+                otherAnimal_y = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][1]
                 if (otherAnimal_x == alt_center_x and otherAnimal_y == alt_center_y) or (otherAnimal_x == alt_assigned_x and otherAnimal_y == alt_assigned_y):
                   print("yeah!!! changed!!", (otherAnimal_x == alt_center_x and otherAnimal_y == alt_center_y), (otherAnimal_x == alt_assigned_x and otherAnimal_y == alt_assigned_y))
-                  output[animal_Id2, i-firstFrame][0][0] = alt_opo_x
-                  output[animal_Id2, i-firstFrame][0][1] = alt_opo_y
+                  trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][0] = alt_opo_x
+                  trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][1] = alt_opo_y
                 break
   
   # Remove doublons
@@ -207,27 +207,27 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
     for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
       # mindist   = 1000000
       # for animal_Id2 in range(0, hyperparameters["nbAnimalsPerWell"]):
-        # x_curFrame_animal_Id   = output[animal_Id, i-firstFrame][0][0]
-        # y_curFrame_animal_Id   = output[animal_Id, i-firstFrame][0][1]
+        # x_curFrame_animal_Id   = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+        # y_curFrame_animal_Id   = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
         # if x_curFrame_animal_Id != 0 and y_curFrame_animal_Id != 0:
           # if animal_Id != animal_Id2:
-            # x_curFrame_animal_Id2  = output[animal_Id2, i-firstFrame][0][0]
-            # y_curFrame_animal_Id2  = output[animal_Id2, i-firstFrame][0][1]
+            # x_curFrame_animal_Id2  = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][0]
+            # y_curFrame_animal_Id2  = trackingHeadTailAllAnimals[animal_Id2, i-firstFrame][0][1]
             # dist = math.sqrt((x_curFrame_animal_Id-x_curFrame_animal_Id2)**2 + (y_curFrame_animal_Id-y_curFrame_animal_Id2)**2)
             # if dist < mindist:
               # mindist = dist
-      mindist = calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, output, i, firstFrame)
+      mindist = calculateMinDistFromOtherAnimals(animal_Id, hyperparameters, trackingHeadTailAllAnimals, i, firstFrame)
       if mindist < 4:
-        print("removed ", output[animal_Id, i-firstFrame][0][0], output[animal_Id, i-firstFrame][0][1])
-        output[animal_Id, i-firstFrame][0][0] = 0
-        output[animal_Id, i-firstFrame][0][1] = 0
+        print("removed ", trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0], trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1])
+        trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] = 0
+        trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] = 0
           
           
   count = 0
   if i > firstFrame:
     for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-      x_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][0]
-      y_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][1]
+      x_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+      y_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
       if x_curFrame_animal_Id != 0 and y_curFrame_animal_Id != 0:
         count = count + 1
   print("Frame ", i, "; count: ", count)
@@ -240,12 +240,12 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
     # maxdepth_min_animal_Id = 0
     # index_animal_Id_alwaysBeenAt0 = -1
     # for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-      # if (output[animal_Id, i-firstFrame][0][0] == 0) and (output[animal_Id, i-firstFrame][0][1] == 0):
+      # if (trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0] == 0) and (trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1] == 0):
         # depth = 1
-        # while (i - firstFrame - depth >= 0) and (depth < 20) and (output[animal_Id, i-firstFrame-depth][0][0] == 0) and (output[animal_Id, i-firstFrame-depth][0][1] == 0):
+        # while (i - firstFrame - depth >= 0) and (depth < 20) and (trackingHeadTailAllAnimals[animal_Id, i-firstFrame-depth][0][0] == 0) and (trackingHeadTailAllAnimals[animal_Id, i-firstFrame-depth][0][1] == 0):
           # depth = depth + 1
-        # xPast_animal_Id = output[animal_Id, i-firstFrame-depth][0][0]
-        # yPast_animal_Id = output[animal_Id, i-firstFrame-depth][0][1]
+        # xPast_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-depth][0][0]
+        # yPast_animal_Id = trackingHeadTailAllAnimals[animal_Id, i-firstFrame-depth][0][1]
         # if (xPast_animal_Id != 0) or (yPast_animal_Id != 0):
           # print("animal_Id:", animal_Id, "; (xPast_animal_Id, yPast_animal_Id):", xPast_animal_Id, yPast_animal_Id)
           # dist = math.sqrt((idx_x_Option - xPast_animal_Id)**2 + (idx_y_Option - yPast_animal_Id)**2)
@@ -261,29 +261,29 @@ def multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, g
             # index_animal_Id_alwaysBeenAt0 = animal_Id
     # print("index_min_animal_Id:", index_min_animal_Id)
     # if index_min_animal_Id >= 0:
-      # output[index_min_animal_Id, i-firstFrame][0][0] = idx_x_Option
-      # output[index_min_animal_Id, i-firstFrame][0][1] = idx_y_Option
-      # x_maxPast_animal_Id = output[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id][0][0]
-      # y_maxPast_animal_Id = output[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id][0][1]
+      # trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame][0][0] = idx_x_Option
+      # trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame][0][1] = idx_y_Option
+      # x_maxPast_animal_Id = trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id][0][0]
+      # y_maxPast_animal_Id = trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id][0][1]
       # print("x_maxPast_animal_Id, y_maxPast_animal_Id:", x_maxPast_animal_Id, y_maxPast_animal_Id)
       # print("maxdepth_min_animal_Id", maxdepth_min_animal_Id)
       # x_step = (idx_x_Option - x_maxPast_animal_Id) / maxdepth_min_animal_Id
       # y_step = (idx_y_Option - y_maxPast_animal_Id) / maxdepth_min_animal_Id
       # for blankSpace in range(1, maxdepth_min_animal_Id):
-        # output[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id+blankSpace][0][0] = x_maxPast_animal_Id + blankSpace * x_step
-        # output[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id+blankSpace][0][1] = y_maxPast_animal_Id + blankSpace * y_step
+        # trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id+blankSpace][0][0] = x_maxPast_animal_Id + blankSpace * x_step
+        # trackingHeadTailAllAnimals[index_min_animal_Id, i-firstFrame-maxdepth_min_animal_Id+blankSpace][0][1] = y_maxPast_animal_Id + blankSpace * y_step
     # else:
       # if index_animal_Id_alwaysBeenAt0 != -1:
-        # output[index_animal_Id_alwaysBeenAt0, i-firstFrame][0][0] = idx_x_Option
-        # output[index_animal_Id_alwaysBeenAt0, i-firstFrame][0][1] = idx_y_Option
+        # trackingHeadTailAllAnimals[index_animal_Id_alwaysBeenAt0, i-firstFrame][0][0] = idx_x_Option
+        # trackingHeadTailAllAnimals[index_animal_Id_alwaysBeenAt0, i-firstFrame][0][1] = idx_y_Option
   
   for animal_Id in range(0, hyperparameters["nbAnimalsPerWell"]):
-    x_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][0]
-    y_curFrame_animal_Id  = output[animal_Id, i-firstFrame][0][1]
+    x_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][0]
+    y_curFrame_animal_Id  = trackingHeadTailAllAnimals[animal_Id, i-firstFrame][0][1]
     if x_curFrame_animal_Id != 0 and y_curFrame_animal_Id != 0:
       x_curFrame_animal_Id = int(x_curFrame_animal_Id)
       y_curFrame_animal_Id = int(y_curFrame_animal_Id)
       heading = 0 #calculateHeadingSimple(x_curFrame_animal_Id, y_curFrame_animal_Id, thresh2, hyperparameters)
-      outputHeading[animal_Id, i-firstFrame] = heading
+      trackingHeadingAllAnimals[animal_Id, i-firstFrame] = heading
   
-  return [outputHeading, output]
+  return [trackingHeadingAllAnimals, trackingHeadTailAllAnimals]
